@@ -11,213 +11,101 @@
 
 using namespace std;
 
-//void boost::assertion_failed(char const * expr, char const * function,
-//                             char const * file, long line)
-//{
-//  fprintf(stdout, "Range check failed for Array please check indices \n" );
-//}
-
-size_t getNextIndex(const vector<size_t> idx, size_t k)
-{
-  if((idx.size()-1)<k)
-    return idx.back();
-  else
-    return idx[k];
-}
-
 /**
 Concatenates n real arrays along the k:th dimension.
 */
 template <typename T>
 void cat_array(int k, const vector<const BaseArray<T>*>& x, BaseArray<T>& a)
 {
-    unsigned int new_k_dim_size = 0;
-    unsigned int n = x.size();
-    /* check dim sizes of all inputs */
-    if(n<1)
-      throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"No input arrays");
+  unsigned int new_k_dim_size = 0;
+  unsigned int n = x.size();
+  /* check dim sizes of all inputs */
+  if (n < 1)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "No input arrays");
 
-    if(x[0]->getDims().size() < k)
-     throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Wrong dimension for input array");
+  if (x[0]->getNumDims() < k)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Wrong dimension for input array");
 
-    new_k_dim_size = x[0]->getDims()[k-1];
-    for(int i = 1; i < n; i++)
-    {
-        //arrays must have same number of dimensions
-		if(x[0]->getDims().size() != x[i]->getDims().size())
-           throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Wrong dimension for input array");
-        //Size matching: Arrays must have identical array sizes with the exception of the size of dimension k
-		for(int j = 0; j < (k - 1); j++)
-        {
-            if (x[0]->getDims()[j] != x[i]->getDims()[j])
-                throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Wrong size for input array");
-        }
+  new_k_dim_size = x[0]->getDim(k);
+  for (int i = 1; i < n; i++)
+  {
+    //arrays must have same number of dimensions
+		if (x[0]->getNumDims() != x[i]->getNumDims())
+      throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Wrong dimension for input array");
+    //Size matching: Arrays must have identical array sizes with the exception of the size of dimension k
+		for (int j = 1; j < k; j++)
+      if (x[0]->getDim(j) != x[i]->getDim(j))
+        throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Wrong size for input array");
 		//calculate new size of dimension k
-        new_k_dim_size += x[i]->getDims()[k-1];
-         //Size matching: Arrays must have identical array sizes with the exception of the size of dimension k
-		for(int j = k; j < x[0]->getDims().size(); j++)
-        {
-          if (x[0]->getDims()[j] != x[i]->getDims()[j])
-            throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Wrong size for input array");
-        }
-    }
-    /* calculate size of sub and super structure in 1-dim data representation */
-    unsigned int n_sub = 1;
-    unsigned int n_super = 1;
-    for (int i = 0; i < (k - 1); i++)
-    {
-        n_super *= x[0]->getDims()[i];
-    }
-    for (int i = k; i < x[0]->getDims().size(); i++)
-    {
-        n_sub *= x[0]->getDims()[i];
-    }
-    /* allocate output array */
-    vector<size_t> ex = x[0]->getDims();
-    ex[k-1] = new_k_dim_size;
-    if(ex.size()<k)
-     throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Error resizing concatenate array");
-    a.setDims( ex );
+    new_k_dim_size += x[i]->getDim(k);
+    //Size matching: Arrays must have identical array sizes with the exception of the size of dimension k
+		for (int j = k + 1; j <= x[0]->getNumDims(); j++)
+      if (x[0]->getDim(j) != x[i]->getDim(j))
+        throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Wrong size for input array");
+  }
+  /* calculate size of sub and super structure in 1-dim data representation */
+  unsigned int n_sub = 1;
+  unsigned int n_super = 1;
+  for (int i = 1; i < k; i++)
+    n_super *= x[0]->getDim(i);
+  for (int i = k + 1; i <= x[0]->getNumDims(); i++)
+    n_sub *= x[0]->getDim(i);
+  /* allocate output array */
+  vector<size_t> ex = x[0]->getDims();
+  ex[k-1] = new_k_dim_size;
+  if (ex.size() < k)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Error resizing concatenate array");
+  a.setDims(ex);
 
   /* concatenation along k-th dimension */
   T* a_data = a.getData();
   int j = 0;
-  for (int i = 0; i < n_super; i++)
+  for (int i = 0; i < n_sub; i++)
   {
     for (int c = 0; c < n; c++)
     {
-      int n_sub_k = n_sub * x[c]->getDims()[k-1];
-      const T* x_data = x[c]->getData();
-      for (int r = 0; r < n_sub_k; r++)
-      {
-        a_data[j] = x_data[r + (i * n_sub_k)];
-        j++;
-      }
+      int n_super_k = n_super * x[c]->getDim(k);
+      const T* x_data = x[c]->getData() + i * n_super_k;
+      std::copy(x_data, x_data + n_super_k, a_data + j);
+      j += n_super_k;
     }
   }
 }
 
-/*
-creates an array (d) for passed multi array  shape (sp) and initialized it with elements from passed source array (s)
-s source array
-d destination array
-sp (shape,indices) of source array
-*/
-template < typename T >
-void create_array_from_shape(const spec_type& sp,BaseArray<T>& s,BaseArray<T>& d)
+void identity_alloc(size_t n, DynArrayDim2<int>& I)
 {
-     //alocate target array
-   vector<size_t> shape;
-     vector<size_t>::const_iterator iter;
-     for(iter = (sp.first).begin();iter!=(sp.first).end();++iter)
-     {
-          if(*iter!=0)
-               shape.push_back(*iter);
-
-     }
-    d.setDims(shape);
-
-     //Check if the dimension of passed indices match the dimension of target array
-   if(sp.second.size()!=s.getNumDims())
-     throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Erro in create array from shape, number of dimensions does not match");
-
-   T* data = new T[d.getNumElems()];
-
-   idx_type::const_iterator spec_iter;
-   //calc number of indeces
-   size_t n =1;
-   for(spec_iter = sp.second.begin();spec_iter!=sp.second.end();++spec_iter)
-     {
-
-        n*=spec_iter->size();
-   }
-   size_t k =0;
-     size_t index=0;
-   vector<size_t>::const_iterator indeces_iter;
-
-   //initialize target array with elements of source array using passed indices
-   vector<size_t> idx;
-   for(int i=0;i<n;i++)
-   {
-    spec_iter = sp.second.begin();
-        for(int dim=0;dim<s.getNumDims();dim++)
-    {
-      size_t idx1 = getNextIndex(*spec_iter,i);
-      idx.push_back(idx1);
-      spec_iter++;
-    }
-    if(index>(d.getNumElems()-1))
-    {
-      throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Erro in create array from shape, number of dimensions does not match");
-    }
-    data[index] = s(idx);
-    idx.clear();
-    index++;
-   }
-   //assign elemets to target array
-   d.assign( data );
-     delete [] data;
+  I.setDims(n, n);
+  fill_array(I, 0);
+  for (size_t i = 1; i <= n; i++)
+    I(i, i) = 1;
 }
 
-
-
-
-/*
-fills an array (d) with passed multi array  shape (sp) and initialized it with elements from passed source array (s)
-s source array
-d destination array
-sp (shape,indices) of source array
-*/
-template < typename T >
-void fill_array_from_shape(const spec_type& sp,BaseArray<T>& s,BaseArray<T>& d)
+template <typename T>
+void diagonal_alloc(const BaseArray<T>& v, BaseArray<T>& D)
 {
-
-
-   T* data = new T[d.getNumElems()];
-
-   idx_type::const_iterator spec_iter;
-   //calc number of indeces
-   size_t n =1;
-   for(spec_iter = sp.second.begin();spec_iter!=sp.second.end();++spec_iter)
-     {
-
-        n*=spec_iter->size();
-   }
-   size_t k =0;
-     size_t index=0;
-   vector<size_t>::const_iterator indeces_iter;
-
-   //initialize target array with elements of source array using passed indices
-   vector<size_t> idx;
-   for(int i=0;i<n;i++)
-   {
-    spec_iter = sp.second.begin();
-        for(int dim=0;dim<s.getNumDims();dim++)
-    {
-      size_t idx1 = getNextIndex(*spec_iter,i);
-      idx.push_back(idx1);
-      spec_iter++;
-    }
-    if(index>(d.getNumElems()-1))
-    {
-      throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"Erro in create array from shape, number of dimensions does not match");
-    }
-    data[index] = s(idx);
-    idx.clear();
-    index++;
-   }
-   //assign elemets to target array
-   d.assign( data );
-     delete [] data;
+  if (v.getNumDims() != 1)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Error in diagonal, input must be vector");
+  if (D.getNumDims() != 2)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "Error in diagonal, output must be matrix");
+  vector<size_t> dims = v.getDims();
+  size_t n = dims[0];
+  dims.push_back(n);
+  D.setDims(dims);
+  const T* v_data = v.getData();
+  T* D_data = D.getData();
+  std::fill(D_data, D_data + n * n, 0);
+  for (size_t i = 0; i < n; i++)
+    D_data[i * n + i] = v_data[i];
 }
 
-
- //template < typename T , size_t NumDims, size_t NumDims2 >
+/**
+ * appends dimensions of size 1 to the right of s up to dimension n
+ */
 template <typename T>
 void promote_array(size_t n, const BaseArray<T>& s, BaseArray<T>& d)
 {
   vector<size_t> ex = s.getDims();
-  for (int i=0; i<n; i++)
+  for (size_t i = ex.size(); i < n; i++)
     ex.push_back(1);
   d.setDims(ex);
   d.assign(s.getData());
@@ -265,12 +153,14 @@ void multiply_array(const BaseArray<T> &leftArray, const BaseArray<T> &rightArra
   size_t leftNumDims = leftArray.getNumDims();
   size_t rightNumDims = rightArray.getNumDims();
   size_t matchDim = rightArray.getDim(1);
-  resultArray.setDims(leftArray.getDims());
   if (leftArray.getDim(leftNumDims) != matchDim)
     throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,
                                   "Wrong sizes in multiply_array");
   if (leftNumDims == 1 && rightNumDims == 2) {
     size_t rightDim = rightArray.getDim(2);
+    vector<size_t> dims;
+    dims.push_back(rightDim);
+    resultArray.setDims(dims);
     for (size_t j = 1; j <= rightDim; j++) {
       T val = T();
       for (size_t k = 1; k <= matchDim; k++)
@@ -280,6 +170,9 @@ void multiply_array(const BaseArray<T> &leftArray, const BaseArray<T> &rightArra
   }
   else if (leftNumDims == 2 && rightNumDims == 1) {
     size_t leftDim = leftArray.getDim(1);
+    vector<size_t> dims;
+    dims.push_back(leftDim);
+    resultArray.setDims(dims);
     for (size_t i = 1; i <= leftDim; i++) {
       T val = T();
       for (size_t k = 1; k <= matchDim; k++)
@@ -290,6 +183,10 @@ void multiply_array(const BaseArray<T> &leftArray, const BaseArray<T> &rightArra
   else if (leftNumDims == 2 && rightNumDims == 2) {
     size_t leftDim = leftArray.getDim(1);
     size_t rightDim = rightArray.getDim(2);
+    vector<size_t> dims;
+    dims.push_back(leftDim);
+    dims.push_back(rightDim);
+    resultArray.setDims(dims);
     for (size_t i = 1; i <= leftDim; i++) {
       for (size_t j = 1; j <= rightDim; j++) {
         T val = T();
@@ -335,6 +232,37 @@ void divide_array(const BaseArray<T>& inputArray, const T &b, BaseArray<T>& outp
   std::transform(data, data + nelems, aim, std::bind2nd(std::divides<T>(), b));
 }
 
+template <typename T>
+void divide_array(const T &b, const BaseArray<T>& inputArray, BaseArray<T>& outputArray)
+{
+  size_t nelems = inputArray.getNumElems();
+  if (outputArray.getNumElems() != nelems)
+  {
+    outputArray.setDims(inputArray.getDims());
+  }
+  const T* data = inputArray.getData();
+  T* aim = outputArray.getData();
+  std::transform(data, data + nelems, aim, std::bind1st(std::divides<T>(), b));
+}
+
+template <typename T>
+void divide_array_elem_wise(const BaseArray<T> &leftArray, const BaseArray<T> &rightArray, BaseArray<T> &resultArray)
+{
+  size_t dimLeft = leftArray.getNumElems();
+  size_t dimRight = rightArray.getNumElems();
+
+  if(dimLeft != dimRight)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,
+      "Right and left array must have the same size for element wise division");
+
+  resultArray.setDims(leftArray.getDims());
+  const T* leftData = leftArray.getData();
+  const T* rightData = rightArray.getData();
+  T* result = resultArray.getData();
+
+  std::transform (leftData, leftData + leftArray.getNumElems(), rightData, result, std::divides<T>());
+}
+
 template <typename T>
 void fill_array(BaseArray<T>& inputArray, T b)
 {
@@ -406,14 +334,29 @@ void add_array(const BaseArray<T>& leftArray, const BaseArray<T>& rightArray, Ba
   std::transform(data1, data1 + leftArray.getNumElems(), data2, aim, std::plus<T>());
 }
 
+template <typename T>
+void add_array_scalar(const BaseArray<T>& inputArray, T b, BaseArray<T>& outputArray)
+{
+  size_t dim = inputArray.getNumElems();
+  if (dim > 0) {
+    outputArray.setDims(inputArray.getDims());
+    const T* data = inputArray.getData();
+    T* result = outputArray.getData();
+    std::transform (data, data + inputArray.getNumElems(),
+                    result, std::bind2nd(std::plus<T>(), b));
+  }
+}
+
 template <typename T>
 void usub_array(const BaseArray<T>& a, BaseArray<T>& b)
 {
-  b.setDims(a.getDims());
-  size_t numEle =  a.getNumElems();
-  for (size_t i = 1; i <= numEle; i++)
-  {
-    b(i) = -a(i);
+  size_t nelems =  a.getNumElems();
+  if (nelems > 0) {
+    b.setDims(a.getDims());
+    const T* data = a.getData();
+    T* result = b.getData();
+    for (size_t i = 0; i < nelems; i++)
+      result[i] = -data[i];
   }
 }
 
@@ -425,15 +368,22 @@ T sum_array (const BaseArray<T>& x)
   return val;
 }
 
+template <typename T>
+T product_array(const BaseArray<T>& x)
+{
+  const T* data = x.getData();
+  T val = std::accumulate(data, data + x.getNumElems(), T(1), std::multiplies<T>());
+  return val;
+}
+
 /**
 scalar product of two arrays (a,b type as template parameter)
 */
 template <typename T>
 T dot_array(const BaseArray<T>& a, const BaseArray<T>& b)
 {
-  if(a.getNumDims() != 1  || b.getNumDims() != 1)
-    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,"error in dot array function. Wrong dimension");
-
+  if (a.getNumDims() != 1  || b.getNumDims() != 1)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "error in dot array function. Wrong dimension");
   const T* data1 = a.getData();
   const T* data2 = b.getData();
   T r = std::inner_product(data1, data1 + a.getNumElems(), data2, 0.0);
@@ -452,10 +402,13 @@ void cross_array(const BaseArray<T>& a, const BaseArray<T>& b, BaseArray<T>& res
 };
 
 /**
-finds min/max elements of an array */
+finds min/max elements of an array
+*/
 template <typename T>
 std::pair<T,T> min_max(const BaseArray<T>& x)
 {
+  if (x.getNumElems() < 1)
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION, "min/max requires at least one element");
   const T* data = x.getData();
   std::pair<const T*, const T*>
   ret = minmax_element(data, data + x.getNumElems());
@@ -553,6 +506,11 @@ transpose_array(const BaseArray<int>& x, BaseArray<int>& a);
 template void BOOST_EXTENSION_EXPORT_DECL
 transpose_array(const BaseArray<bool>& x, BaseArray<bool>& a);
 
+template void BOOST_EXTENSION_EXPORT_DECL
+diagonal_alloc(const BaseArray<double>& v, BaseArray<double>& D);
+template void BOOST_EXTENSION_EXPORT_DECL
+diagonal_alloc(const BaseArray<int>& v, BaseArray<int>& D);
+
 template void BOOST_EXTENSION_EXPORT_DECL
 promote_array(size_t n, const BaseArray<double>& s, BaseArray<double>& d);
 template void BOOST_EXTENSION_EXPORT_DECL
@@ -560,15 +518,6 @@ promote_array(size_t n, const BaseArray<int>& s, BaseArray<int>& d);
 template void BOOST_EXTENSION_EXPORT_DECL
 promote_array(size_t n, const BaseArray<bool>& s, BaseArray<bool>& d);
 
-template void BOOST_EXTENSION_EXPORT_DECL create_array_from_shape(const spec_type& sp, BaseArray<double>& s, BaseArray<double>& d);
-template void BOOST_EXTENSION_EXPORT_DECL create_array_from_shape(const spec_type& sp, BaseArray<int>& s, BaseArray<int>& d);
-template void BOOST_EXTENSION_EXPORT_DECL create_array_from_shape(const spec_type& sp, BaseArray<bool>& s, BaseArray<bool>& d);
-
-
-template void BOOST_EXTENSION_EXPORT_DECL fill_array_from_shape(const spec_type& sp, BaseArray<double>& s, BaseArray<double>& d);
-template void BOOST_EXTENSION_EXPORT_DECL fill_array_from_shape(const spec_type& sp, BaseArray<int>& s, BaseArray<int>& d);
-template void BOOST_EXTENSION_EXPORT_DECL fill_array_from_shape(const spec_type& sp, BaseArray<bool>& s, BaseArray<bool>& d);
-
 template void BOOST_EXTENSION_EXPORT_DECL
 multiply_array(const BaseArray<double>& inputArray, const double &b, BaseArray<double>& outputArray);
 template void BOOST_EXTENSION_EXPORT_DECL
@@ -597,6 +546,18 @@ divide_array(const BaseArray<int>& inputArray, const int &b, BaseArray<int>& out
 template void BOOST_EXTENSION_EXPORT_DECL
 divide_array(const BaseArray<bool>& inputArray, const bool &b, BaseArray<bool>& outputArray);
 
+template void BOOST_EXTENSION_EXPORT_DECL
+divide_array(const double &b, const BaseArray<double>& inputArray, BaseArray<double>& outputArray);
+template void BOOST_EXTENSION_EXPORT_DECL
+divide_array(const int &b, const BaseArray<int>& inputArray, BaseArray<int>& outputArray);
+
+template void BOOST_EXTENSION_EXPORT_DECL
+divide_array_elem_wise(const BaseArray<double> &leftArray, const BaseArray<double> &rightArray, BaseArray<double> &resultArray);
+template void BOOST_EXTENSION_EXPORT_DECL
+divide_array_elem_wise(const BaseArray<int> &leftArray, const BaseArray<int> &rightArray, BaseArray<int> &resultArray);
+template void BOOST_EXTENSION_EXPORT_DECL
+divide_array_elem_wise(const BaseArray<bool> &leftArray, const BaseArray<bool> &rightArray, BaseArray<bool> &resultArray);
+
 template void BOOST_EXTENSION_EXPORT_DECL
 fill_array(BaseArray<double>& inputArray, double b);
 template void BOOST_EXTENSION_EXPORT_DECL
@@ -630,6 +591,13 @@ add_array(const BaseArray<int>& leftArray, const BaseArray<int>& rightArray, Bas
 template void BOOST_EXTENSION_EXPORT_DECL
 add_array(const BaseArray<bool>& leftArray, const BaseArray<bool>& rightArray, BaseArray<bool>& resultArray);
 
+template void BOOST_EXTENSION_EXPORT_DECL
+add_array_scalar(const BaseArray<double>& inputArray, double b, BaseArray<double>& outputArray);
+template void BOOST_EXTENSION_EXPORT_DECL
+add_array_scalar(const BaseArray<int>& inputArray, int b, BaseArray<int>& outputArray);
+template void BOOST_EXTENSION_EXPORT_DECL
+add_array_scalar(const BaseArray<bool>& inputArray, bool b, BaseArray<bool>& outputArray);
+
 template void BOOST_EXTENSION_EXPORT_DECL
 usub_array(const BaseArray<double>& a, BaseArray<double>& b);
 template void BOOST_EXTENSION_EXPORT_DECL
@@ -644,6 +612,11 @@ sum_array(const BaseArray<int>& x);
 template bool BOOST_EXTENSION_EXPORT_DECL
 sum_array(const BaseArray<bool>& x);
 
+template double BOOST_EXTENSION_EXPORT_DECL
+product_array(const BaseArray<double>& x);
+template int BOOST_EXTENSION_EXPORT_DECL
+product_array(const BaseArray<int>& x);
+
 template void BOOST_EXTENSION_EXPORT_DECL
 cross_array(const BaseArray<double>& a, const BaseArray<double>& b, BaseArray<double>& res);
 template void BOOST_EXTENSION_EXPORT_DECL
