@@ -35,14 +35,15 @@
 #include "Util.h"
 #include "Variable.h"
 
-#include <iostream>
-#include <string>
-#include <fstream>
-#include <sstream>
-#include <stdlib.h>
-#include <stack>
 #include <algorithm>
 #include <deque>
+#include <fstream>
+#include <iostream>
+#include <set>
+#include <sstream>
+#include <stack>
+#include <stdlib.h>
+#include <string>
 
 oms::DirectedGraph::DirectedGraph()
 {
@@ -58,7 +59,7 @@ void oms::DirectedGraph::clear()
   G.clear();
   sortedConnections.clear();
   nodes.clear();
-  edges.clear();
+  edges.connections.clear();
   sortedConnectionsAreValid = true;
 }
 
@@ -92,7 +93,7 @@ void oms::DirectedGraph::addEdge(const oms::Connector& var1, const oms::Connecto
   if (-1 == index2)
     index2 = addNode(var2);
 
-  edges.push_back(std::pair<int, int>(index1, index2));
+  edges.connections.push_back(std::pair<int, int>(index1, index2));
   G[index1].push_back(index2);
   sortedConnectionsAreValid = false;
 }
@@ -124,10 +125,10 @@ void oms::DirectedGraph::dotExport(const std::string& filename)
   }
   dotFile << std::endl;
 
-  for (int i = 0; i < edges.size(); i++)
+  for (int i = 0; i < edges.connections.size(); i++)
   {
-    dotFile << "  " << edges[i].first << " -> " << edges[i].second;
-    if (nodes[edges[i].first].isOutput() && nodes[edges[i].second].isInput())
+    dotFile << "  " << edges.connections[i].first << " -> " << edges.connections[i].second;
+    if (nodes[edges.connections[i].first].isOutput() && nodes[edges.connections[i].second].isInput())
       dotFile << " [color=\"red\"];" << std::endl;
     else
       dotFile << std::endl;
@@ -141,14 +142,14 @@ void oms::DirectedGraph::includeGraph(const oms::DirectedGraph& graph, const oms
   for (int i = 0; i < graph.nodes.size(); i++)
     addNode(graph.nodes[i].addPrefix(prefix));
 
-  for (int i = 0; i < graph.edges.size(); i++)
-    addEdge(graph.nodes[graph.edges[i].first].addPrefix(prefix), graph.nodes[graph.edges[i].second].addPrefix(prefix));
+  for (int i = 0; i < graph.edges.connections.size(); i++)
+    addEdge(graph.nodes[graph.edges.connections[i].first].addPrefix(prefix), graph.nodes[graph.edges.connections[i].second].addPrefix(prefix));
 }
 
-int oms::DirectedGraph::getEdgeIndex(const std::vector< std::pair<int, int> >& edges, int from, int to)
+int oms::DirectedGraph::getEdgeIndex(const scc_t& edges, int from, int to)
 {
-  for (int i = 0; i < edges.size(); ++i)
-    if (edges[i].first == from && edges[i].second == to)
+  for (int i = 0; i < edges.connections.size(); ++i)
+    if (edges.connections[i].first == from && edges.connections[i].second == to)
       return i;
 
   logError("getEdgeIndex failed");
@@ -165,10 +166,10 @@ void oms::DirectedGraph::strongconnect(int v, std::vector< std::vector<int> > G,
   stacked[v] = true;
 
   // Consider successors of v
-  std::vector<int> successors = G[edges[v].second];
+  std::vector<int> successors = G[edges.connections[v].second];
   for (int i = 0; i < successors.size(); ++i)
   {
-    int w = getEdgeIndex(edges, edges[v].second, successors[i]);
+    int w = getEdgeIndex(edges, edges.connections[v].second, successors[i]);
     if (d[w] == -1)
     {
       // Successor w has not yet been visited; recurse on it
@@ -207,7 +208,7 @@ std::deque< std::vector<int> > oms::DirectedGraph::getSCCs()
 {
   //std::cout << "Tarjan's strongly connected components algorithm" << std::endl;
 
-  size_t numVertices = edges.size();
+  size_t numVertices = edges.connections.size();
   int *d = new int[numVertices];
   std::fill(d, d + numVertices, -1);
   int *low = new int[numVertices];
@@ -243,7 +244,7 @@ std::deque< std::vector<int> > oms::DirectedGraph::getSCCs()
   return components;
 }
 
-const std::vector< std::vector< std::pair<int, int> > >& oms::DirectedGraph::getSortedConnections()
+const std::vector<oms::scc_t>& oms::DirectedGraph::getSortedConnections()
 {
   if (!sortedConnectionsAreValid)
     calculateSortedConnections();
@@ -253,27 +254,122 @@ const std::vector< std::vector< std::pair<int, int> > >& oms::DirectedGraph::get
 void oms::DirectedGraph::calculateSortedConnections()
 {
   std::deque< std::vector<int> > components = getSCCs();
-  std::vector< std::pair<int, int> > SCC;
   sortedConnections.clear();
 
   for (int i = 0; i < components.size(); ++i)
   {
-    SCC.clear();
+    scc_t scc;
     for (int j = 0; j < components[i].size(); ++j)
     {
-      Connector conA = nodes[edges[components[i][j]].first];
-      Connector conB = nodes[edges[components[i][j]].second];
+      Connector conA = nodes[edges.connections[components[i][j]].first];
+      Connector conB = nodes[edges.connections[components[i][j]].second];
 
       if (oms::Connection::isValid(conA.getName(), conB.getName(), conA, conB))
-        SCC.push_back(std::pair<int, int>(edges[components[i][j]]));
+      {
+        scc.connections.push_back(std::pair<int, int>(edges.connections[components[i][j]]));
+        scc.component_names.insert(conA.getOwner());
+        scc.component_names.insert(conB.getOwner());
+        // check for factor in connector units
+        if (nodes[edges.connections[components[i][j]].first].connectorUnits.empty() || nodes[edges.connections[components[i][j]].second].connectorUnits.empty())
+          scc.factor = 1.0;
+        else
+        {
+          double factorA = 1.0;
+          double factorB = 1.0;
+          for (const auto & connectorA : nodes[edges.connections[components[i][j]].first].connectorUnits)
+          {
+            for (const auto &baseunit : connectorA.second)
+            {
+              if (baseunit.first == "factor")
+                factorA = std::stod(baseunit.second);
+            }
+            //std::cout << "\n factorA ==> " << conA.getName().c_str() << "==>" << connectorA.first << "==>" << factorA;
+          }
+          for (const auto &connectorB : nodes[edges.connections[components[i][j]].second].connectorUnits)
+          {
+            for (const auto &baseunit : connectorB.second)
+            {
+              if (baseunit.first == "factor")
+                factorB = std::stod(baseunit.second);
+            }
+            //std::cout << "\n factorB ==> " << conB.getName().c_str() << "==>" << connectorB.first << "==>" << factorB;
+          }
+          // factor = output_Connector_Factor/ input_Connector_Factor
+          scc.factor = (factorA/factorB);
+
+          // set suppressUnitConversion = true or false
+          scc.suppressUnitConversion = false;
+          for (const auto &it : unitConversion)
+          {
+            if (it.conA == conA.getName() && it.conB == conB.getName())
+            {
+              scc.suppressUnitConversion = it.unitConversion;
+              break;
+            }
+          }
+        }
+      }
     }
 
-    if (SCC.size() > 0)
-      sortedConnections.push_back(SCC);
 
-    if (SCC.size() > 1)
-      logWarning("Alg. loop (size " + std::to_string(SCC.size()) + ")");
+    // size of loop incl. internal connections: components[i].size()
+    // size of loop excl. internal connections: connections.size()
+    scc.thisIsALoop = (components[i].size() > 1);
+    scc.size = scc.connections.size();
+    scc.size_including_internal = components[i].size();
+
+    if (scc.size > 0)
+    {
+      if (scc.thisIsALoop)
+      {
+        std::stringstream ss;
+        ss << "Alg. loop (size " << scc.size << "/" << scc.size_including_internal << ")" << std::endl;
+        for (const auto& name: scc.component_names)
+          ss << "  " << std::string(name) << std::endl;
+        logInfo(ss.str());
+      }
+      sortedConnections.push_back(scc);
+    }
   }
 
   sortedConnectionsAreValid = true;
 }
+
+void oms::DirectedGraph::setUnits(Connector* conA, Connector* conB, bool suppressUnitConversion)
+{
+  /* get the full cref to check the connector owner with nodes
+     (e.g) model.root.A.y1 ==> A.y1
+  */
+  ComRef crefA(conA->getOwner() + conA->getName());
+  ComRef tailA = crefA.pop_front();
+  ComRef tailB = crefA.pop_front();
+
+  ComRef crefB(conB->getOwner() + conB->getName());
+  ComRef tailA1 = crefB.pop_front();
+  ComRef tailB1 = crefB.pop_front();
+
+  unitConversion.push_back({crefA, crefB, suppressUnitConversion});
+
+  for (auto &it : nodes)
+  {
+    // std::cout << "\n after edge:" << it.getName().c_str() << "==>" << crefA.c_str() << "==>" << crefB.c_str();
+    if (it.getName() == crefA)
+    {
+      for (const auto &con : conA->connectorUnits)
+        it.connectorUnits[con.first] = con.second;
+    }
+    if (it.getName() == crefB)
+    {
+      for (const auto &con : conB->connectorUnits)
+        it.connectorUnits[con.first] = con.second;
+    }
+  }
+}
+
+void oms::DirectedGraph::dumpNodes() const
+{
+  std::string msg = "";
+  for (int i = 0; i < nodes.size(); ++i)
+    msg += std::to_string(i+1) + ": " + std::string(nodes[i].getName()) + "\n";
+  logInfo(msg);
+}
